@@ -311,6 +311,16 @@ class JSONOptions(CodecOptions):
                     self.json_mode,
                     super(JSONOptions, self)._arguments_repr()))
 
+    def _options_dict(self):
+        # TODO: PYTHON-2442 use _asdict() instead
+        options_dict = super(JSONOptions, self)._options_dict()
+        options_dict.update({
+            'strict_number_long': self.strict_number_long,
+            'datetime_representation': self.datetime_representation,
+            'strict_uuid': self.strict_uuid,
+            'json_mode': self.json_mode})
+        return options_dict
+
     def with_options(self, **kwargs):
         """
         Make a copy of this JSONOptions, overriding some options::
@@ -324,7 +334,7 @@ class JSONOptions(CodecOptions):
 
         .. versionadded:: 3.12
         """
-        opts = self._asdict()
+        opts = self._options_dict()
         for opt in ('strict_number_long', 'datetime_representation',
                     'strict_uuid', 'json_mode'):
             opts[opt] = kwargs.get(opt, getattr(self, opt))
@@ -495,7 +505,7 @@ def object_hook(dct, json_options=DEFAULT_JSON_OPTIONS):
 def _parse_legacy_regex(doc):
     pattern = doc["$regex"]
     # Check if this is the $regex query operator.
-    if isinstance(pattern, Regex):
+    if not isinstance(pattern, (text_type, bytes)):
         return doc
     flags = 0
     # PyMongo always adds $options but some other tools may not.
