@@ -23,12 +23,37 @@ from pymongo import ASCENDING
 from pymongo.errors import (CursorNotFound,
                             DuplicateKeyError,
                             ExecutionTimeout,
-                            NotMasterError,
+                            NotPrimaryError,
                             OperationFailure,
                             WriteError,
                             WriteConcernError,
                             WTimeoutError)
+from pymongo.hello_compat import HelloCompat
 
+# From the SDAM spec, the "node is shutting down" codes.
+_SHUTDOWN_CODES = frozenset([
+    11600,  # InterruptedAtShutdown
+    91,     # ShutdownInProgress
+])
+# From the SDAM spec, the "not primary" error codes are combined with the
+# "node is recovering" error codes (of which the "node is shutting down"
+# errors are a subset).
+_NOT_MASTER_CODES = frozenset([
+    10058,  # LegacyNotPrimary <=3.2 "not primary" error code
+    10107,  # NotWritablePrimary
+    13435,  # NotPrimaryNoSecondaryOk
+    11602,  # InterruptedDueToReplStateChange
+    13436,  # NotPrimaryOrSecondary
+    189,    # PrimarySteppedDown
+]) | _SHUTDOWN_CODES
+# From the retryable writes spec.
+_RETRYABLE_ERROR_CODES = _NOT_MASTER_CODES | frozenset([
+    7,     # HostNotFound
+    6,     # HostUnreachable
+    89,    # NetworkTimeout
+    9001,  # SocketException
+    262,   # ExceededTimeLimit
+])
 _UUNDER = u"_"
 
 
@@ -74,13 +99,13 @@ def _index_document(index_list):
             raise TypeError("first item in each key pair must be a string")
         if not isinstance(value, (string_type, int, abc.Mapping)):
             raise TypeError("second item in each key pair must be 1, -1, "
-                            "'2d', 'geoHaystack', or another valid MongoDB "
-                            "index specifier.")
+                            "'2d', or another valid MongoDB index specifier.")
         index[key] = value
     return index
 
 
-def _check_command_response(response, msg=None, allowable_errors=None,
+def _check_command_response(response, max_wire_version,
+                            allowable_errors=None,
                             parse_write_concern_error=False):
     """Check the response to a command for errors.
     """
@@ -88,67 +113,64 @@ def _check_command_response(response, msg=None, allowable_errors=None,
         # Server didn't recognize our message as a command.
         raise OperationFailure(response.get("$err"),
                                response.get("code"),
-                               response)
-
-    # TODO: remove, this is moving to _check_gle_response
-    if response.get("wtimeout", False):
-        # MongoDB versions before 1.8.0 return the error message in an "errmsg"
-        # field. If "errmsg" exists "err" will also exist set to None, so we
-        # have to check for "errmsg" first.
-        raise WTimeoutError(response.get("errmsg", response.get("err")),
-                            response.get("code"),
-                            response)
+                               response,
+                               max_wire_version)
 
     if parse_write_concern_error and 'writeConcernError' in response:
-        wce = response['writeConcernError']
-        raise WriteConcernError(wce['errmsg'], wce['code'], wce)
+        _error = response["writeConcernError"]
+        _labels = response.get("errorLabels")
+        if _labels:
+            _error.update({'errorLabels': _labels})
+        _raise_write_concern_error(_error)
 
-    if not response["ok"]:
+    if response["ok"]:
+        return
 
-        details = response
-        # Mongos returns the error details in a 'raw' object
-        # for some errors.
-        if "raw" in response:
-            for shard in itervalues(response["raw"]):
-                # Grab the first non-empty raw error from a shard.
-                if shard.get("errmsg") and not shard.get("ok"):
-                    details = shard
-                    break
+    details = response
+    # Mongos returns the error details in a 'raw' object
+    # for some errors.
+    if "raw" in response:
+        for shard in itervalues(response["raw"]):
+            # Grab the first non-empty raw error from a shard.
+            if shard.get("errmsg") and not shard.get("ok"):
+                details = shard
+                break
 
-        errmsg = details["errmsg"]
-        if allowable_errors is None or errmsg not in allowable_errors:
+    errmsg = details["errmsg"]
+    code = details.get("code")
 
-            # Server is "not master" or "recovering"
-            if (errmsg.startswith("not master")
-                    or errmsg.startswith("node is recovering")):
-                raise NotMasterError(errmsg, response)
+    # For allowable errors, only check for error messages when the code is not
+    # included.
+    if allowable_errors:
+        if code is not None:
+            if code in allowable_errors:
+                return
+        elif errmsg in allowable_errors:
+            return
 
-            # Server assertion failures
-            if errmsg == "db assertion failure":
-                errmsg = ("db assertion failure, assertion: '%s'" %
-                          details.get("assertion", ""))
-                raise OperationFailure(errmsg,
-                                       details.get("assertionCode"),
-                                       response)
+    # Server is "not primary" or "recovering"
+    if code is not None:
+        if code in _NOT_MASTER_CODES:
+            raise NotPrimaryError(errmsg, response)
+    elif HelloCompat.LEGACY_ERROR in errmsg or "node is recovering" in errmsg:
+        raise NotPrimaryError(errmsg, response)
 
-            # Other errors
-            code = details.get("code")
-            # findAndModify with upsert can raise duplicate key error
-            if code in (11000, 11001, 12582):
-                raise DuplicateKeyError(errmsg, code, response)
-            elif code == 50:
-                raise ExecutionTimeout(errmsg, code, response)
-            elif code == 43:
-                raise CursorNotFound(errmsg, code, response)
+    # Other errors
+    # findAndModify with upsert can raise duplicate key error
+    if code in (11000, 11001, 12582):
+        raise DuplicateKeyError(errmsg, code, response, max_wire_version)
+    elif code == 50:
+        raise ExecutionTimeout(errmsg, code, response, max_wire_version)
+    elif code == 43:
+        raise CursorNotFound(errmsg, code, response, max_wire_version)
 
-            msg = msg or "%s"
-            raise OperationFailure(msg % errmsg, code, response)
+    raise OperationFailure(errmsg, code, response, max_wire_version)
 
 
-def _check_gle_response(result):
+def _check_gle_response(result, max_wire_version):
     """Return getlasterror response as a dict, or raise OperationFailure."""
     # Did getlasterror itself fail?
-    _check_command_response(result)
+    _check_command_response(result, max_wire_version)
 
     if result.get("wtimeout", False):
         # MongoDB versions before 1.8.0 return the error message in an "errmsg"
@@ -162,8 +184,8 @@ def _check_gle_response(result):
     if error_msg is None:
         return result
 
-    if error_msg.startswith("not master"):
-        raise NotMasterError(error_msg, result)
+    if error_msg.startswith(HelloCompat.LEGACY_ERROR):
+        raise NotPrimaryError(error_msg, result)
 
     details = result
 
@@ -198,6 +220,18 @@ def _raise_write_concern_error(error):
         error.get("errmsg"), error.get("code"), error)
 
 
+def _get_wce_doc(result):
+    """Return the writeConcernError or None."""
+    wce = result.get("writeConcernError")
+    if wce:
+        # The server reports errorLabels at the top level but it's more
+        # convenient to attach it to the writeConcernError doc itself.
+        error_labels = result.get("errorLabels")
+        if error_labels:
+            wce["errorLabels"] = error_labels
+    return wce
+
+
 def _check_write_command_response(result):
     """Backward compatibility helper for write command error handling.
     """
@@ -206,9 +240,9 @@ def _check_write_command_response(result):
     if write_errors:
         _raise_last_write_error(write_errors)
 
-    error = result.get("writeConcernError")
-    if error:
-        _raise_write_concern_error(error)
+    wce = _get_wce_doc(result)
+    if wce:
+        _raise_write_concern_error(wce)
 
 
 def _raise_last_error(bulk_write_result):
