@@ -95,6 +95,7 @@ if CA_PEM:
 COMPRESSORS = os.environ.get("COMPRESSORS")
 MONGODB_API_VERSION = os.environ.get("MONGODB_API_VERSION")
 TEST_LOADBALANCER = bool(os.environ.get("TEST_LOADBALANCER"))
+TEST_SERVERLESS = bool(os.environ.get("TEST_SERVERLESS"))
 SINGLE_MONGOS_LB_URI = os.environ.get("SINGLE_MONGOS_LB_URI")
 MULTI_MONGOS_LB_URI = os.environ.get("MULTI_MONGOS_LB_URI")
 if TEST_LOADBALANCER:
@@ -105,6 +106,15 @@ if TEST_LOADBALANCER:
     host, port = res['nodelist'][0]
     db_user = res['username'] or db_user
     db_pwd = res['password'] or db_pwd
+elif TEST_SERVERLESS:
+    TEST_LOADBALANCER = True
+    res = parse_uri(SINGLE_MONGOS_LB_URI)
+    host, port = res['nodelist'][0]
+    db_user = res['username'] or db_user
+    db_pwd = res['password'] or db_pwd
+    TLS_OPTIONS = {'tls': True}
+    # Spec says serverless tests must be run with compression.
+    COMPRESSORS = COMPRESSORS or 'zlib'
 
 
 def is_server_resolvable():
@@ -218,7 +228,7 @@ class ClientContext(object):
         self.version = Version(-1)  # Needs to be comparable with Version
         self.auth_enabled = False
         self.test_commands_enabled = False
-        self.server_parameters = None
+        self.server_parameters = {}
         self.is_mongos = False
         self.mongoses = []
         self.is_rs = False
@@ -232,7 +242,8 @@ class ClientContext(object):
         self.conn_lock = threading.Lock()
         self.is_data_lake = False
         self.load_balancer = TEST_LOADBALANCER
-        if self.load_balancer:
+        self.serverless = TEST_SERVERLESS
+        if self.load_balancer or self.serverless:
             self.default_client_options["loadBalanced"] = True
         if COMPRESSORS:
             self.default_client_options["compressors"] = COMPRESSORS
@@ -299,22 +310,26 @@ class ClientContext(object):
         if self.client:
             self.connected = True
 
-            try:
-                self.cmd_line = self.client.admin.command('getCmdLineOpts')
-            except pymongo.errors.OperationFailure as e:
-                msg = e.details.get('errmsg', '')
-                if e.code == 13 or 'unauthorized' in msg or 'login' in msg:
-                    # Unauthorized.
-                    self.auth_enabled = True
-                else:
-                    raise
+            if self.serverless:
+                self.auth_enabled = True
             else:
-                self.auth_enabled = self._server_started_with_auth()
+                try:
+                    self.cmd_line = self.client.admin.command('getCmdLineOpts')
+                except pymongo.errors.OperationFailure as e:
+                    msg = e.details.get('errmsg', '')
+                    if e.code == 13 or 'unauthorized' in msg or 'login' in msg:
+                        # Unauthorized.
+                        self.auth_enabled = True
+                    else:
+                        raise
+                else:
+                    self.auth_enabled = self._server_started_with_auth()
 
             if self.auth_enabled:
-                # See if db_user already exists.
-                if not self._check_user_provided():
-                    _create_user(self.client.admin, db_user, db_pwd)
+                if not self.serverless:
+                    # See if db_user already exists.
+                    if not self._check_user_provided():
+                        _create_user(self.client.admin, db_user, db_pwd)
 
                 self.client = self._connect(
                     host, port, username=db_user, password=db_pwd,
@@ -324,10 +339,13 @@ class ClientContext(object):
                 # May not have this if OperationFailure was raised earlier.
                 self.cmd_line = self.client.admin.command('getCmdLineOpts')
 
-            self.server_status = self.client.admin.command('serverStatus')
-            if self.storage_engine == "mmapv1":
-                # MMAPv1 does not support retryWrites=True.
-                self.default_client_options['retryWrites'] = False
+            if self.serverless:
+                self.server_status = {}
+            else:
+                self.server_status = self.client.admin.command('serverStatus')
+                if self.storage_engine == "mmapv1":
+                    # MMAPv1 does not support retryWrites=True.
+                    self.default_client_options['retryWrites'] = False
 
             hello = self.hello
             self.sessions_enabled = 'logicalSessionTimeoutMinutes' in hello
@@ -364,33 +382,42 @@ class ClientContext(object):
                 self.nodes = set([(host, port)])
             self.w = len(hello.get("hosts", [])) or 1
             self.version = Version.from_client(self.client)
-            self.server_parameters = self.client.admin.command(
-                'getParameter', '*')
 
-            if 'enableTestCommands=1' in self.cmd_line['argv']:
+            if self.serverless:
+                self.server_parameters = {
+                    'requireApiVersion': False,
+                    'enableTestCommands': True,
+                }
                 self.test_commands_enabled = True
-            elif 'parsed' in self.cmd_line:
-                params = self.cmd_line['parsed'].get('setParameter', [])
-                if 'enableTestCommands=1' in params:
+                self.has_ipv6 = False
+            else:
+                self.server_parameters = self.client.admin.command(
+                    'getParameter', '*')
+                if 'enableTestCommands=1' in self.cmd_line['argv']:
                     self.test_commands_enabled = True
-                else:
-                    params = self.cmd_line['parsed'].get('setParameter', {})
-                    if params.get('enableTestCommands') == '1':
+                elif 'parsed' in self.cmd_line:
+                    params = self.cmd_line['parsed'].get('setParameter', [])
+                    if 'enableTestCommands=1' in params:
                         self.test_commands_enabled = True
+                    else:
+                        params = self.cmd_line['parsed'].get('setParameter', {})
+                        if params.get('enableTestCommands') == '1':
+                            self.test_commands_enabled = True
+                    self.has_ipv6 = self._server_started_with_ipv6()
 
             self.is_mongos = (self.hello.get('msg') == 'isdbgrid')
-            self.has_ipv6 = self._server_started_with_ipv6()
             if self.is_mongos:
-                # Check for another mongos on the next port.
                 address = self.client.address
-                next_address = address[0], address[1] + 1
                 self.mongoses.append(address)
-                mongos_client = self._connect(*next_address,
-                                              **self.default_client_options)
-                if mongos_client:
-                    hello = mongos_client.admin.command(HelloCompat.LEGACY_CMD)
-                    if hello.get('msg') == 'isdbgrid':
-                        self.mongoses.append(next_address)
+                if not self.serverless:
+                    # Check for another mongos on the next port.
+                    next_address = address[0], address[1] + 1
+                    mongos_client = self._connect(
+                        *next_address, **self.default_client_options)
+                    if mongos_client:
+                        hello = mongos_client.admin.command(HelloCompat.LEGACY_CMD)
+                        if hello.get('msg') == 'isdbgrid':
+                            self.mongoses.append(next_address)
 
     def init(self):
         with self.conn_lock:
@@ -878,6 +905,9 @@ class IntegrationTest(PyMongoTestCase):
         if (client_context.load_balancer and
                 not getattr(cls, 'RUN_ON_LOAD_BALANCER', False)):
             raise SkipTest('this test does not support load balancers')
+        if (client_context.serverless and
+                not getattr(cls, 'RUN_ON_SERVERLESS', False)):
+            raise SkipTest('this test does not support serverless')
         cls.client = client_context.client
         cls.db = cls.client.pymongo_test
         if client_context.auth_enabled:
@@ -1045,6 +1075,9 @@ class SystemCertsPatcher(object):
             raise SkipTest(
                 "LibreSSL on OSX doesn't support setting CA certificates "
                 "using SSL_CERT_FILE environment variable.")
+        if sys.platform == 'win32' and _ssl.IS_PYOPENSSL:
+            raise SkipTest(
+                "SSL_CERT_FILE does not work on Windows with PyOpenSSL")
         self.original_certs = os.environ.get('SSL_CERT_FILE')
         # Tell OpenSSL where CA certificates live.
         os.environ['SSL_CERT_FILE'] = ca_certs
