@@ -32,14 +32,14 @@ from pymongo.hello_compat import HelloCompat
 from pymongo.errors import OperationFailure
 from pymongo.read_preferences import ReadPreference
 from pymongo.saslprep import HAVE_STRINGPREP
-from test import client_context, SkipTest, unittest, Version
+from test import client_context, IntegrationTest, SkipTest, unittest, Version
 from test.utils import (delay,
                         ignore_deprecations,
                         single_client,
                         rs_or_single_client,
                         rs_or_single_client_noauth,
                         single_client_noauth,
-                        WhiteListEventListener)
+                        AllowListEventListener)
 
 # YOU MUST RUN KINIT BEFORE RUNNING GSSAPI TESTS ON UNIX.
 GSSAPI_HOST = os.environ.get('GSSAPI_HOST')
@@ -351,11 +351,12 @@ class TestSASLPlain(unittest.TestCase):
         self.assertRaises(OperationFailure, bad_pwd.admin.command, HelloCompat.LEGACY_CMD)
 
 
-class TestSCRAMSHA1(unittest.TestCase):
+class TestSCRAMSHA1(IntegrationTest):
 
     @client_context.require_auth
     @client_context.require_version_min(2, 7, 2)
     def setUp(self):
+        super(TestSCRAMSHA1, self).setUp()
         # Before 2.7.7, SCRAM-SHA-1 had to be enabled from the command line.
         if client_context.version < Version(2, 7, 7):
             cmd_line = client_context.cmd_line
@@ -369,6 +370,7 @@ class TestSCRAMSHA1(unittest.TestCase):
 
     def tearDown(self):
         client_context.drop_user('pymongo_test', 'user')
+        super(TestSCRAMSHA1, self).tearDown()
 
     def test_scram_sha1(self):
         host, port = client_context.host, client_context.port
@@ -396,22 +398,24 @@ class TestSCRAMSHA1(unittest.TestCase):
             db.command('dbstats')
 
 
-class TestSCRAM(unittest.TestCase):
+class TestSCRAM(IntegrationTest):
 
     @client_context.require_auth
     @client_context.require_version_min(3, 7, 2)
     def setUp(self):
+        super(TestSCRAM, self).setUp()
         self._SENSITIVE_COMMANDS = monitoring._SENSITIVE_COMMANDS
         monitoring._SENSITIVE_COMMANDS = set([])
-        self.listener = WhiteListEventListener("saslStart")
+        self.listener = AllowListEventListener("saslStart")
 
     def tearDown(self):
         monitoring._SENSITIVE_COMMANDS = self._SENSITIVE_COMMANDS
         client_context.client.testscram.command("dropAllUsersFromDatabase")
         client_context.client.drop_database("testscram")
+        super(TestSCRAM, self).tearDown()
 
     def test_scram_skip_empty_exchange(self):
-        listener = WhiteListEventListener("saslStart", "saslContinue")
+        listener = AllowListEventListener("saslStart", "saslContinue")
         client_context.create_user(
             'testscram', 'sha256', 'pwd', roles=['dbOwner'],
             mechanisms=['SCRAM-SHA-256'])
@@ -656,7 +660,8 @@ class TestSCRAM(unittest.TestCase):
             thread.join()
             self.assertTrue(thread.success)
 
-class TestThreadedAuth(unittest.TestCase):
+
+class TestThreadedAuth(IntegrationTest):
 
     @client_context.require_auth
     def test_db_authenticate_threaded(self):
@@ -689,10 +694,11 @@ class TestThreadedAuth(unittest.TestCase):
             self.assertTrue(thread.success)
 
 
-class TestAuthURIOptions(unittest.TestCase):
+class TestAuthURIOptions(IntegrationTest):
 
     @client_context.require_auth
     def setUp(self):
+        super(TestAuthURIOptions, self).setUp()
         client_context.create_user('admin', 'admin', 'pass')
         client_context.create_user(
             'pymongo_test', 'user', 'pass', ['userAdmin', 'readWrite'])
@@ -700,6 +706,7 @@ class TestAuthURIOptions(unittest.TestCase):
     def tearDown(self):
         client_context.drop_user('pymongo_test', 'user')
         client_context.drop_user('admin', 'admin')
+        super(TestAuthURIOptions, self).tearDown()
 
     def test_uri_options(self):
         # Test default to admin
