@@ -28,7 +28,8 @@ TEST_PATH = os.path.join(
     os.path.dirname(os.path.realpath(__file__)), 'crud', 'unified')
 
 # Generate unified tests.
-globals().update(generate_test_classes(TEST_PATH, module=__name__))
+globals().update(generate_test_classes(
+    TEST_PATH, module=__name__, RUN_ON_SERVERLESS=True))
 
 if __name__ == "__main__":
     unittest.main()
