@@ -41,13 +41,15 @@ from pymongo.errors import (ConfigurationError,
                             OperationFailure)
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
+from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   unittest,
                   IntegrationTest)
 from test.utils import (EventListener,
+                        OvertCommandListener,
                         ignore_deprecations,
                         rs_or_single_client,
-                        WhiteListEventListener)
+                        AllowListEventListener)
 
 if PY3:
     long = int
@@ -237,7 +239,7 @@ class TestCursor(IntegrationTest):
                 10).max_await_time_ms(90)
         self.assertEqual(90, cursor._Cursor__max_await_time_ms)
 
-        listener = WhiteListEventListener('find', 'getMore')
+        listener = AllowListEventListener('find', 'getMore')
         coll = rs_or_single_client(
             event_listeners=[listener])[self.db.name].pymongo_test
         results = listener.results
@@ -355,7 +357,7 @@ class TestCursor(IntegrationTest):
 
     def test_explain_with_read_concern(self):
         # Do not add readConcern level to explain.
-        listener = WhiteListEventListener("explain")
+        listener = AllowListEventListener("explain")
         client = rs_or_single_client(event_listeners=[listener])
         self.addCleanup(client.close)
         coll = client.pymongo_test.test.with_options(
@@ -1396,7 +1398,7 @@ class TestCursor(IntegrationTest):
         gc.collect()
         self.client._process_periodic_tasks()
 
-        listener = WhiteListEventListener("killCursors")
+        listener = AllowListEventListener("killCursors")
         results = listener.results
         client = rs_or_single_client(event_listeners=[listener])
         self.addCleanup(client.close)
@@ -1443,7 +1445,7 @@ class TestCursor(IntegrationTest):
 
     @client_context.require_version_min(3, 6)
     def test_getMore_does_not_send_readPreference(self):
-        listener = WhiteListEventListener('find', 'getMore')
+        listener = AllowListEventListener('find', 'getMore')
         client = rs_or_single_client(
             event_listeners=[listener])
         self.addCleanup(client.close)
@@ -1477,13 +1479,93 @@ class TestRawBatchCursor(IntegrationTest):
         with self.assertRaises(InvalidOperation):
             c.find_raw_batches(manipulate=True)
 
+    @client_context.require_transactions
+    def test_find_raw_transaction(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        with client.start_session() as session:
+            with session.start_transaction():
+                batches = list(client[self.db.name].test.find_raw_batches(
+                    session=session).sort('_id'))
+                cmd = listener.results['started'][0]
+                self.assertEqual(cmd.command_name, 'find')
+                self.assertIn('$clusterTime', cmd.command)
+                self.assertEqual(cmd.command['startTransaction'], True)
+                self.assertEqual(cmd.command['txnNumber'], 1)
+                # Ensure we update $clusterTime from the command response.
+                last_cmd = listener.results['succeeded'][-1]
+                self.assertEqual(last_cmd.reply['$clusterTime']['clusterTime'],
+                                 session.cluster_time['clusterTime'])
+
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+
+    @client_context.require_sessions
+    @client_context.require_failCommand_fail_point
+    def test_find_raw_retryable_reads(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener],
+                                     retryReads=True)
+        with self.fail_point({
+            'mode': {'times': 1}, 'data': {'failCommands': ['find'],
+                                           'closeConnection': True}}):
+            batches = list(
+                client[self.db.name].test.find_raw_batches().sort('_id'))
+
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+        self.assertEqual(len(listener.results['started']), 2)
+        for cmd in listener.results['started']:
+            self.assertEqual(cmd.command_name, 'find')
+
+    @client_context.require_version_min(5, 0, 0)
+    @client_context.require_no_standalone
+    def test_find_raw_snapshot_reads(self):
+        c = self.db.get_collection(
+            "test", write_concern=WriteConcern(w="majority"))
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener],
+                                     retryReads=True)
+        db = client[self.db.name]
+        with client.start_session(snapshot=True) as session:
+            db.test.distinct('x', {}, session=session)
+            batches = list(db.test.find_raw_batches(
+                session=session).sort('_id'))
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+
+        find_cmd = listener.results['started'][1].command
+        self.assertEqual(find_cmd['readConcern']['level'], 'snapshot')
+        self.assertIsNotNone(find_cmd['readConcern']['atClusterTime'])
+
     def test_explain(self):
         c = self.db.test
         c.insert_one({})
         explanation = c.find_raw_batches().explain()
         self.assertIsInstance(explanation, dict)
 
+    def test_empty(self):
+        self.db.test.drop()
+        cursor = self.db.test.find_raw_batches()
+        with self.assertRaises(StopIteration):
+            next(cursor)
+
     def test_clone(self):
+        self.db.test.insert_one({})
         cursor = self.db.test.find_raw_batches()
         # Copy of a RawBatchCursor is also a RawBatchCursor, not a Cursor.
         self.assertIsInstance(next(cursor.clone()), bytes)
@@ -1518,7 +1600,10 @@ class TestRawBatchCursor(IntegrationTest):
             next(self.db.test.find_raw_batches(collation=Collation('en_US')))
 
     @client_context.require_version_min(3, 2)
+    @client_context.require_no_mmap   # MMAPv1 does not support read concern
     def test_read_concern(self):
+        self.db.get_collection(
+            "test", write_concern=WriteConcern(w="majority")).insert_one({})
         c = self.db.get_collection("test", read_concern=ReadConcern("majority"))
         next(c.find_raw_batches())
 
@@ -1591,6 +1676,79 @@ class TestRawBatchCommandCursor(IntegrationTest):
         self.assertEqual(1, len(batches))
         self.assertEqual(docs, decode_all(batches[0]))
 
+    @client_context.require_transactions
+    def test_aggregate_raw_transaction(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        with client.start_session() as session:
+            with session.start_transaction():
+                batches = list(client[self.db.name].test.aggregate_raw_batches(
+                    [{'$sort': {'_id': 1}}], session=session))
+                cmd = listener.results['started'][0]
+                self.assertEqual(cmd.command_name, 'aggregate')
+                self.assertIn('$clusterTime', cmd.command)
+                self.assertEqual(cmd.command['startTransaction'], True)
+                self.assertEqual(cmd.command['txnNumber'], 1)
+                # Ensure we update $clusterTime from the command response.
+                last_cmd = listener.results['succeeded'][-1]
+                self.assertEqual(last_cmd.reply['$clusterTime']['clusterTime'],
+                                 session.cluster_time['clusterTime'])
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+
+    @client_context.require_sessions
+    @client_context.require_failCommand_fail_point
+    def test_aggregate_raw_retryable_reads(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener],
+                                     retryReads=True)
+        with self.fail_point({
+            'mode': {'times': 1}, 'data': {'failCommands': ['aggregate'],
+                                           'closeConnection': True}}):
+            batches = list(client[self.db.name].test.aggregate_raw_batches(
+                [{'$sort': {'_id': 1}}]))
+
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+        self.assertEqual(len(listener.results['started']), 3)
+        cmds = listener.results['started']
+        self.assertEqual(cmds[0].command_name, 'aggregate')
+        self.assertEqual(cmds[1].command_name, 'aggregate')
+
+    @client_context.require_version_min(5, 0, -1)
+    @client_context.require_no_standalone
+    def test_aggregate_raw_snapshot_reads(self):
+        c = self.db.get_collection(
+            "test", write_concern=WriteConcern(w="majority"))
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener],
+                                     retryReads=True)
+        db = client[self.db.name]
+        with client.start_session(snapshot=True) as session:
+            db.test.distinct('x', {}, session=session)
+            batches = list(db.test.aggregate_raw_batches(
+                [{'$sort': {'_id': 1}}], session=session))
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+
+        find_cmd = listener.results['started'][1].command
+        self.assertEqual(find_cmd['readConcern']['level'], 'snapshot')
+        self.assertIsNotNone(find_cmd['readConcern']['atClusterTime'])
+
     def test_server_error(self):
         c = self.db.test
         c.drop()
