@@ -58,6 +58,7 @@ from test.utils import (EventListener,
                         rs_or_single_client,
                         server_started_with_auth,
                         wait_until,
+                        DeprecationFilter,
                         IMPOSSIBLE_WRITE_CONCERN,
                         OvertCommandListener)
 from test.test_custom_types import DECIMAL_CODECOPTS
@@ -126,6 +127,9 @@ class TestDatabase(IntegrationTest):
         self.assertFalse(Database(self.client, "test") !=
                          Database(self.client, "test"))
 
+    def test_hashable(self):
+        self.assertIn(self.client.test, {Database(self.client, "test")})
+
     def test_get_coll(self):
         db = Database(self.client, "pymongo_test")
         self.assertEqual(db.test, db["test"])
@@ -387,6 +391,7 @@ class TestDatabase(IntegrationTest):
                 db.validate_collection(coll, full=True, background=True)
 
     @client_context.require_no_mongos
+    @ignore_deprecations
     def test_profiling_levels(self):
         db = self.client.pymongo_test
         self.assertEqual(db.profiling_level(), OFF)  # default
@@ -416,6 +421,38 @@ class TestDatabase(IntegrationTest):
         self.assertEqual(100, db.command("profile", -1)['slowms'])
 
     @client_context.require_no_mongos
+    @client_context.require_version_min(3, 6)
+    @ignore_deprecations
+    def test_profiling_sample_rate(self):
+        db = self.client.pymongo_test
+        with self.assertRaises(TypeError):
+            db.set_profiling_level(SLOW_ONLY, 50, sample_rate='1')
+        with self.assertRaises(TypeError):
+            db.set_profiling_level(SLOW_ONLY, 50, sample_rate=1)
+
+        db.set_profiling_level(SLOW_ONLY, 50, sample_rate=0.0)
+        db.set_profiling_level(SLOW_ONLY, 50, sample_rate=1.0)
+        db.set_profiling_level(SLOW_ONLY, 50, sample_rate=0.5)
+        profile = db.command("profile", -1)
+        self.assertEqual(50, profile['slowms'])
+        self.assertEqual(0.5, profile['sampleRate'])
+        db.set_profiling_level(OFF, 100)  # back to default
+        self.assertEqual(100, db.command("profile", -1)['slowms'])
+
+    @client_context.require_no_mongos
+    @client_context.require_version_min(4, 4, 2)
+    @ignore_deprecations
+    def test_profiling_filter(self):
+        db = self.client.pymongo_test
+        db.set_profiling_level(ALL, filter={'ns': {'$eq': 'test.test'}})
+        profile = db.command("profile", -1)
+        self.assertEqual({'ns': {'$eq': 'test.test'}}, profile['filter'])
+        # filter='unset' resets the filter back to the default.
+        db.set_profiling_level(OFF, 100, filter='unset')
+        self.assertEqual(100, db.command("profile", -1)['slowms'])
+
+    @client_context.require_no_mongos
+    @ignore_deprecations
     def test_profiling_info(self):
         db = self.client.pymongo_test
 
@@ -444,6 +481,16 @@ class TestDatabase(IntegrationTest):
         self.assertTrue(isinstance(info[0]['op'], string_type))
         self.assertTrue(isinstance(info[0]["ts"], datetime.datetime))
 
+    def test_profiling_helpers_deprecated(self):
+        filter = DeprecationFilter('error')
+        self.addCleanup(filter.stop)
+        db = self.client.pymongo_test
+        self.assertRaises(DeprecationWarning, db.profiling_level)
+        self.assertRaises(DeprecationWarning, db.profiling_info)
+        self.assertRaises(DeprecationWarning, db.set_profiling_level, OFF)
+
+    # SERVER-47817 removes the resetError command.
+    @client_context.require_version_max(4, 9)
     @client_context.require_no_mongos
     @ignore_deprecations
     def test_errors(self):
