@@ -52,6 +52,34 @@ class TestJsonUtil(unittest.TestCase):
     def test_basic(self):
         self.round_trip({"hello": "world"})
 
+    def test_json_options_with_options(self):
+        opts = json_util.JSONOptions(
+            datetime_representation=DatetimeRepresentation.NUMBERLONG)
+        self.assertEqual(
+            opts.datetime_representation, DatetimeRepresentation.NUMBERLONG)
+        opts2 = opts.with_options(
+            datetime_representation=DatetimeRepresentation.ISO8601)
+        self.assertEqual(
+            opts2.datetime_representation, DatetimeRepresentation.ISO8601)
+
+        opts = json_util.JSONOptions(strict_number_long=True)
+        self.assertEqual(opts.strict_number_long, True)
+        opts2 = opts.with_options(strict_number_long=False)
+        self.assertEqual(opts2.strict_number_long, False)
+
+        opts = json_util.CANONICAL_JSON_OPTIONS
+        self.assertNotEqual(
+            opts.uuid_representation, UuidRepresentation.JAVA_LEGACY)
+        opts2 = opts.with_options(
+            uuid_representation=UuidRepresentation.JAVA_LEGACY)
+        self.assertEqual(
+            opts2.uuid_representation, UuidRepresentation.JAVA_LEGACY)
+        self.assertEqual(opts2.document_class, dict)
+        opts3 = opts2.with_options(document_class=SON)
+        self.assertEqual(
+            opts3.uuid_representation, UuidRepresentation.JAVA_LEGACY)
+        self.assertEqual(opts3.document_class, SON)
+
     def test_objectid(self):
         self.round_trip({"id": ObjectId()})
 
@@ -234,6 +262,15 @@ class TestJsonUtil(unittest.TestCase):
             '{"$regex": ".*", "$options": "mx"}',
             json_util.dumps(re.compile(b'.*', re.M | re.X)))
 
+    def test_regex_validation(self):
+        non_str_types = [10, {}, []]
+        docs = [{"$regex": i} for i in non_str_types]
+        for doc in docs:
+            self.assertEqual(doc, json_util.loads(json.dumps(doc)))
+
+        doc = {"$regex": ""}
+        self.assertIsInstance(json_util.loads(json.dumps(doc)), Regex)
+
     def test_minkey(self):
         self.round_trip({"m": MinKey()})
 
