@@ -20,19 +20,20 @@ import warnings
 
 sys.path[0:0] = [""]
 
+from bson.int64 import Int64
 from bson.objectid import ObjectId
 from bson.py3compat import text_type
 from bson.son import SON
 from pymongo import CursorType, monitoring, InsertOne, UpdateOne, DeleteOne
 from pymongo.command_cursor import CommandCursor
 from pymongo.errors import (AutoReconnect,
-                            NotMasterError,
+                            NotPrimaryError,
                             OperationFailure)
 from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   client_knobs,
-                  PyMongoTestCase,
+                  IntegrationTest,
                   sanitize_cmd,
                   unittest)
 from test.utils import (EventListener,
@@ -43,11 +44,12 @@ from test.utils import (EventListener,
                         wait_until)
 
 
-class TestCommandMonitoring(PyMongoTestCase):
+class TestCommandMonitoring(IntegrationTest):
 
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
+        super(TestCommandMonitoring, cls).setUpClass()
         cls.listener = EventListener()
         cls.client = rs_or_single_client(
             event_listeners=[cls.listener],
@@ -56,12 +58,14 @@ class TestCommandMonitoring(PyMongoTestCase):
     @classmethod
     def tearDownClass(cls):
         cls.client.close()
+        super(TestCommandMonitoring, cls).tearDownClass()
 
     def tearDown(self):
         self.listener.results.clear()
+        super(TestCommandMonitoring, self).tearDown()
 
     def test_started_simple(self):
-        self.client.pymongo_test.command('ismaster')
+        self.client.pymongo_test.command('ping')
         results = self.listener.results
         started = results['started'][0]
         succeeded = results['succeeded'][0]
@@ -70,14 +74,14 @@ class TestCommandMonitoring(PyMongoTestCase):
             isinstance(succeeded, monitoring.CommandSucceededEvent))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqualCommand(SON([('ismaster', 1)]), started.command)
-        self.assertEqual('ismaster', started.command_name)
+        self.assertEqualCommand(SON([('ping', 1)]), started.command)
+        self.assertEqual('ping', started.command_name)
         self.assertEqual(self.client.address, started.connection_id)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertTrue(isinstance(started.request_id, int))
 
     def test_succeeded_simple(self):
-        self.client.pymongo_test.command('ismaster')
+        self.client.pymongo_test.command('ping')
         results = self.listener.results
         started = results['started'][0]
         succeeded = results['succeeded'][0]
@@ -86,7 +90,7 @@ class TestCommandMonitoring(PyMongoTestCase):
             isinstance(started, monitoring.CommandStartedEvent))
         self.assertTrue(
             isinstance(succeeded, monitoring.CommandSucceededEvent))
-        self.assertEqual('ismaster', succeeded.command_name)
+        self.assertEqual('ping', succeeded.command_name)
         self.assertEqual(self.client.address, succeeded.connection_id)
         self.assertEqual(1, succeeded.reply.get('ok'))
         self.assertTrue(isinstance(succeeded.request_id, int))
@@ -399,7 +403,8 @@ class TestCommandMonitoring(PyMongoTestCase):
     def test_get_more_failure(self):
         address = self.client.address
         coll = self.client.pymongo_test.test
-        cursor_doc = {"id": 12345, "firstBatch": [], "ns": coll.full_name}
+        cursor_id = Int64(12345)
+        cursor_doc = {"id": cursor_id, "firstBatch": [], "ns": coll.full_name}
         cursor = CommandCursor(coll, cursor_doc, address)
         try:
             next(cursor)
@@ -412,7 +417,7 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
         self.assertEqualCommand(
-            SON([('getMore', 12345),
+            SON([('getMore', cursor_id),
                  ('collection', 'test')]),
             started.command)
         self.assertEqual('getMore', started.command_name)
@@ -429,16 +434,16 @@ class TestCommandMonitoring(PyMongoTestCase):
 
     @client_context.require_replica_set
     @client_context.require_secondaries_count(1)
-    def test_not_master_error(self):
+    def test_not_primary_error(self):
         address = next(iter(client_context.client.secondaries))
         client = single_client(*address, event_listeners=[self.listener])
         # Clear authentication command results from the listener.
-        client.admin.command('ismaster')
+        client.admin.command('ping')
         self.listener.results.clear()
         error = None
         try:
             client.pymongo_test.test.find_one_and_delete({})
-        except NotMasterError as exc:
+        except NotPrimaryError as exc:
             error = exc.errors
         results = self.listener.results
         started = results['started'][0]
@@ -458,7 +463,7 @@ class TestCommandMonitoring(PyMongoTestCase):
     @client_context.require_no_mongos
     def test_exhaust(self):
         self.client.pymongo_test.test.drop()
-        self.client.pymongo_test.test.insert_many([{} for _ in range(10)])
+        self.client.pymongo_test.test.insert_many([{} for _ in range(11)])
         self.listener.results.clear()
         cursor = self.client.pymongo_test.test.find(
             projection={'_id': False},
@@ -472,12 +477,10 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqualCommand(
-            SON([('find', 'test'),
-                 ('filter', {}),
-                 ('projection', {'_id': False}),
-                 ('batchSize', 5)]),
-            started.command)
+        self.assertEqualCommand(SON([('find', 'test'),
+                                     ('filter', {}),
+                                     ('projection', {'_id': False}),
+                                     ('batchSize', 5)]), started.command)
         self.assertEqual('find', started.command_name)
         self.assertEqual(cursor.address, started.connection_id)
         self.assertEqual('pymongo_test', started.database_name)
@@ -498,32 +501,25 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.listener.results.clear()
         tuple(cursor)
         results = self.listener.results
-        started = results['started'][0]
-        succeeded = results['succeeded'][0]
         self.assertEqual(0, len(results['failed']))
-        self.assertTrue(
-            isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqualCommand(
-            SON([('getMore', cursor_id),
-                 ('collection', 'test'),
-                 ('batchSize', 5)]),
-            started.command)
-        self.assertEqual('getMore', started.command_name)
-        self.assertEqual(cursor.address, started.connection_id)
-        self.assertEqual('pymongo_test', started.database_name)
-        self.assertTrue(isinstance(started.request_id, int))
-        self.assertTrue(
-            isinstance(succeeded, monitoring.CommandSucceededEvent))
-        self.assertTrue(isinstance(succeeded.duration_micros, int))
-        self.assertEqual('getMore', succeeded.command_name)
-        self.assertTrue(isinstance(succeeded.request_id, int))
-        self.assertEqual(cursor.address, succeeded.connection_id)
-        expected_result = {
-            'cursor': {'id': 0,
-                       'ns': 'pymongo_test.test',
-                       'nextBatch': [{} for _ in range(5)]},
-            'ok': 1}
-        self.assertEqualReply(expected_result, succeeded.reply)
+        for event in results['started']:
+            self.assertTrue(isinstance(event, monitoring.CommandStartedEvent))
+            self.assertEqualCommand(SON([('getMore', cursor_id),
+                                         ('collection', 'test'),
+                                         ('batchSize', 5)]), event.command)
+            self.assertEqual('getMore', event.command_name)
+            self.assertEqual(cursor.address, event.connection_id)
+            self.assertEqual('pymongo_test', event.database_name)
+            self.assertTrue(isinstance(event.request_id, int))
+        for event in results['succeeded']:
+            self.assertTrue(
+                isinstance(event, monitoring.CommandSucceededEvent))
+            self.assertTrue(isinstance(event.duration_micros, int))
+            self.assertEqual('getMore', event.command_name)
+            self.assertTrue(isinstance(event.request_id, int))
+            self.assertEqual(cursor.address, event.connection_id)
+        # Last getMore receives a response with cursor id 0.
+        self.assertEqual(0, results['succeeded'][-1].reply['cursor']['id'])
 
     def test_kill_cursors(self):
         with client_knobs(kill_cursor_frequency=0.01):
@@ -1215,11 +1211,11 @@ class TestCommandMonitoring(PyMongoTestCase):
             'data': {
                 'failCommands': ['insert'],
                 'closeConnection': False,
-                'errorCode': 10107,  # NotMaster
+                'errorCode': 10107,  # NotPrimary
             },
         }
         with self.fail_point(insert_command_error):
-            with self.assertRaises(NotMasterError):
+            with self.assertRaises(NotPrimaryError):
                 coll.bulk_write([InsertOne({'_id': 1})])
         failed = self.listener.results['failed']
         self.assertEqual(1, len(failed))
@@ -1391,29 +1387,32 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertEqual({}, succeeded.reply)
 
 
-class TestGlobalListener(PyMongoTestCase):
+class TestGlobalListener(IntegrationTest):
 
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
+        super(TestGlobalListener, cls).setUpClass()
         cls.listener = EventListener()
         # We plan to call register(), which internally modifies _LISTENERS.
         cls.saved_listeners = copy.deepcopy(monitoring._LISTENERS)
         monitoring.register(cls.listener)
         cls.client = single_client()
         # Get one (authenticated) socket in the pool.
-        cls.client.pymongo_test.command('ismaster')
+        cls.client.pymongo_test.command('ping')
 
     @classmethod
     def tearDownClass(cls):
         monitoring._LISTENERS = cls.saved_listeners
         cls.client.close()
+        super(TestGlobalListener, cls).tearDownClass()
 
     def setUp(self):
+        super(TestGlobalListener, self).setUp()
         self.listener.results.clear()
 
     def test_simple(self):
-        self.client.pymongo_test.command('ismaster')
+        self.client.pymongo_test.command('ping')
         results = self.listener.results
         started = results['started'][0]
         succeeded = results['succeeded'][0]
@@ -1422,39 +1421,40 @@ class TestGlobalListener(PyMongoTestCase):
             isinstance(succeeded, monitoring.CommandSucceededEvent))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqualCommand(SON([('ismaster', 1)]), started.command)
-        self.assertEqual('ismaster', started.command_name)
+        self.assertEqualCommand(SON([('ping', 1)]), started.command)
+        self.assertEqual('ping', started.command_name)
         self.assertEqual(self.client.address, started.connection_id)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertTrue(isinstance(started.request_id, int))
 
 
-class TestEventClasses(PyMongoTestCase):
+class TestEventClasses(unittest.TestCase):
 
     def test_command_event_repr(self):
         request_id, connection_id, operation_id = 1, ('localhost', 27017), 2
         event = monitoring.CommandStartedEvent(
-            {'isMaster': 1}, 'admin', request_id, connection_id, operation_id)
+            {'ping': 1}, 'admin', request_id, connection_id, operation_id)
         self.assertEqual(
             repr(event),
             "<CommandStartedEvent ('localhost', 27017) db: 'admin', "
-            "command: 'isMaster', operation_id: 2>")
+            "command: 'ping', operation_id: 2, service_id: None>")
         delta = datetime.timedelta(milliseconds=100)
         event = monitoring.CommandSucceededEvent(
-            delta, {'ok': 1}, 'isMaster', request_id, connection_id,
+            delta, {'ok': 1}, 'ping', request_id, connection_id,
             operation_id)
         self.assertEqual(
             repr(event),
             "<CommandSucceededEvent ('localhost', 27017) "
-            "command: 'isMaster', operation_id: 2, duration_micros: 100000>")
+            "command: 'ping', operation_id: 2, duration_micros: 100000, "
+            "service_id: None>")
         event = monitoring.CommandFailedEvent(
-            delta, {'ok': 0}, 'isMaster', request_id, connection_id,
+            delta, {'ok': 0}, 'ping', request_id, connection_id,
             operation_id)
         self.assertEqual(
             repr(event),
             "<CommandFailedEvent ('localhost', 27017) "
-            "command: 'isMaster', operation_id: 2, duration_micros: 100000, "
-            "failure: {'ok': 0}>")
+            "command: 'ping', operation_id: 2, duration_micros: 100000, "
+            "failure: {'ok': 0}, service_id: None>")
 
     def test_server_heartbeat_event_repr(self):
         connection_id = ('localhost', 27017)
