@@ -50,11 +50,12 @@ from test.version import Version
 
 # Location of JSON test specifications.
 _TEST_PATH = os.path.join(
-    os.path.dirname(os.path.realpath(__file__)), 'retryable_writes')
+    os.path.dirname(os.path.realpath(__file__)), 'retryable_writes', 'legacy')
 
 
 class TestAllScenarios(SpecRunner):
     RUN_ON_LOAD_BALANCER = True
+    RUN_ON_SERVERLESS = True
 
     def get_object_name(self, op):
         return op.get('object', 'collection')
@@ -165,6 +166,7 @@ def non_retryable_single_statement_ops(coll):
 
 class IgnoreDeprecationsTest(IntegrationTest):
     RUN_ON_LOAD_BALANCER = True
+    RUN_ON_SERVERLESS = True
 
     @classmethod
     def setUpClass(cls):
@@ -462,6 +464,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
 
 class TestWriteConcernError(IntegrationTest):
     RUN_ON_LOAD_BALANCER = True
+    RUN_ON_SERVERLESS = True
 
     @classmethod
     @client_context.require_replica_set
@@ -528,6 +531,7 @@ class TestRetryableWritesTxnNumber(IgnoreDeprecationsTest):
         listener = OvertCommandListener()
         client = rs_or_single_client(
             retryWrites=True, event_listeners=[listener])
+        self.addCleanup(client.close)
         topology = client._topology
         select_server = topology.select_server
 
