@@ -18,23 +18,26 @@ import sys
 
 sys.path[0:0] = [""]
 
+from bson.objectid import ObjectId
 from bson.py3compat import imap
+
 from pymongo import common
-from pymongo.read_preferences import ReadPreference, Secondary
-from pymongo.server_type import SERVER_TYPE
-from pymongo.topology import (_ErrorContext,
-                              Topology)
-from pymongo.topology_description import TOPOLOGY_TYPE
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
                             ConnectionFailure)
-from pymongo.ismaster import IsMaster
+from pymongo.hello import Hello
+from pymongo.hello_compat import HelloCompat
 from pymongo.monitor import Monitor
 from pymongo.pool import PoolOptions
+from pymongo.read_preferences import ReadPreference, Secondary
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
+from pymongo.server_type import SERVER_TYPE
 from pymongo.settings import TopologySettings
+from pymongo.topology import (_ErrorContext,
+                              Topology)
+from pymongo.topology_description import TOPOLOGY_TYPE
 from test import client_knobs, unittest
 from test.utils import MockPool, wait_until
 
@@ -81,9 +84,9 @@ def create_mock_topology(
     return t
 
 
-def got_ismaster(topology, server_address, ismaster_response):
+def got_hello(topology, server_address, hello_response):
     server_description = ServerDescription(
-        server_address, IsMaster(ismaster_response), 0)
+        server_address, Hello(hello_response), 0)
 
     topology.on_change(server_description)
 
@@ -135,23 +138,23 @@ class TestTopologyConfiguration(TopologyTest):
         self.assertEqual(1, monitor._pool.opts.connect_timeout)
         self.assertEqual(1, monitor._pool.opts.socket_timeout)
 
-        # The monitor, not its pool, is responsible for calling ismaster.
+        # The monitor, not its pool, is responsible for calling hello.
         self.assertFalse(monitor._pool.handshake)
 
 
 class TestSingleServerTopology(TopologyTest):
     def test_direct_connection(self):
-        for server_type, ismaster_response in [
+        for server_type, hello_response in [
             (SERVER_TYPE.RSPrimary, {
                 'ok': 1,
-                'ismaster': True,
+                HelloCompat.LEGACY_CMD: True,
                 'hosts': ['a'],
                 'setName': 'rs',
                 'maxWireVersion': 6}),
 
             (SERVER_TYPE.RSSecondary, {
                 'ok': 1,
-                'ismaster': False,
+                HelloCompat.LEGACY_CMD: False,
                 'secondary': True,
                 'hosts': ['a'],
                 'setName': 'rs',
@@ -159,13 +162,13 @@ class TestSingleServerTopology(TopologyTest):
 
             (SERVER_TYPE.Mongos, {
                 'ok': 1,
-                'ismaster': True,
+                HelloCompat.LEGACY_CMD: True,
                 'msg': 'isdbgrid',
                 'maxWireVersion': 6}),
 
             (SERVER_TYPE.RSArbiter, {
                 'ok': 1,
-                'ismaster': False,
+                HelloCompat.LEGACY_CMD: False,
                 'arbiterOnly': True,
                 'hosts': ['a'],
                 'setName': 'rs',
@@ -173,13 +176,14 @@ class TestSingleServerTopology(TopologyTest):
 
             (SERVER_TYPE.Standalone, {
                 'ok': 1,
-                'ismaster': True,
+                HelloCompat.LEGACY_CMD: True,
                 'maxWireVersion': 6}),
 
-            # Slave.
+            # mongod started as "secondary" in the replication system
+            # that existed before replica sets.
             (SERVER_TYPE.Standalone, {
                 'ok': 1,
-                'ismaster': False,
+                HelloCompat.LEGACY_CMD: False,
                 'maxWireVersion': 6}),
         ]:
             t = create_mock_topology()
@@ -190,7 +194,7 @@ class TestSingleServerTopology(TopologyTest):
                 t.select_servers(any_server_selector,
                                  server_selection_timeout=0)
 
-            got_ismaster(t, address, ismaster_response)
+            got_hello(t, address, hello_response)
 
             # Topology type never changes.
             self.assertEqual(TOPOLOGY_TYPE.Single, t.description.topology_type)
@@ -228,7 +232,7 @@ class TestSingleServerTopology(TopologyTest):
         class TestMonitor(Monitor):
             def _check_with_socket(self, *args, **kwargs):
                 if available:
-                    return (IsMaster({'ok': 1, 'maxWireVersion': 6}),
+                    return (Hello({'ok': 1, 'maxWireVersion': 6}),
                             round_trip_time)
                 else:
                     raise AutoReconnect('mock monitor error')
@@ -281,20 +285,20 @@ class TestSingleServerTopology(TopologyTest):
 class TestMultiServerTopology(TopologyTest):
     def test_readable_writable(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        self.assertTrue(
+        self.assertEqual(
             t.description.topology_type_name, 'ReplicaSetWithPrimary')
         self.assertTrue(t.description.has_writable_server())
         self.assertTrue(t.description.has_readable_server())
@@ -305,21 +309,21 @@ class TestMultiServerTopology(TopologyTest):
                 Secondary(tag_sets=[{'tag': 'exists'}])))
 
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': False,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        self.assertTrue(
+        self.assertEqual(
             t.description.topology_type_name, 'ReplicaSetNoPrimary')
         self.assertFalse(t.description.has_writable_server())
         self.assertFalse(t.description.has_readable_server())
@@ -330,21 +334,21 @@ class TestMultiServerTopology(TopologyTest):
                 Secondary(tag_sets=[{'tag': 'exists'}])))
 
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
             'tags': {'tag': 'exists'}})
 
-        self.assertTrue(
+        self.assertEqual(
             t.description.topology_type_name, 'ReplicaSetWithPrimary')
         self.assertTrue(t.description.has_writable_server())
         self.assertTrue(t.description.has_readable_server())
@@ -356,15 +360,15 @@ class TestMultiServerTopology(TopologyTest):
 
     def test_close(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
@@ -386,10 +390,10 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
                          t.description.topology_type)
 
-        # A closed topology should not be updated when receiving an isMaster.
-        got_ismaster(t, ('a', 27017), {
+        # A closed topology should not be updated when receiving an hello.
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b', 'c']})
 
@@ -406,20 +410,20 @@ class TestMultiServerTopology(TopologyTest):
 
     def test_handle_error(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        errctx = _ErrorContext(AutoReconnect('mock'), 0, 0, True)
+        errctx = _ErrorContext(AutoReconnect('mock'), 0, 0, True, None)
         t.handle_error(('a', 27017), errctx)
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.RSSecondary, get_type(t, 'b'))
@@ -427,9 +431,9 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
                          t.description.topology_type)
 
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
@@ -446,29 +450,29 @@ class TestMultiServerTopology(TopologyTest):
 
     def test_handle_getlasterror(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        t.handle_getlasterror(('a', 27017), 'not master')
+        t.handle_getlasterror(('a', 27017), HelloCompat.LEGACY_ERROR)
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.RSSecondary, get_type(t, 'b'))
         self.assertEqual('rs', t.description.replica_set_name)
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
                          t.description.topology_type)
 
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
@@ -480,7 +484,7 @@ class TestMultiServerTopology(TopologyTest):
         t = create_mock_topology(replica_set_name='rs')
 
         # No error resetting a server not in the TopologyDescription.
-        errctx = _ErrorContext(AutoReconnect('mock'), 0, 0, True)
+        errctx = _ErrorContext(AutoReconnect('mock'), 0, 0, True, None)
         t.handle_error(('b', 27017), errctx)
 
         # Server was *not* added as type Unknown.
@@ -490,7 +494,7 @@ class TestMultiServerTopology(TopologyTest):
         t = create_mock_topology(replica_set_name='rs')
 
         # No error resetting a server not in the TopologyDescription.
-        t.handle_getlasterror(('b', 27017), 'not master')
+        t.handle_getlasterror(('b', 27017), HelloCompat.LEGACY_ERROR)
 
         # Server was *not* added as type Unknown.
         self.assertFalse(t.has_server(('b', 27017)))
@@ -508,9 +512,9 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(t.description.topology_type,
                          TOPOLOGY_TYPE.ReplicaSetNoPrimary)
         t.open()
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a']})
 
@@ -520,9 +524,9 @@ class TestMultiServerTopology(TopologyTest):
 
         # Another response from the primary. Tests the code that processes
         # primary response when topology type is already ReplicaSetWithPrimary.
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a']})
 
@@ -544,9 +548,9 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(t.description.topology_type,
                          TOPOLOGY_TYPE.ReplicaSetNoPrimary)
         t.open()
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a']})
@@ -559,9 +563,9 @@ class TestMultiServerTopology(TopologyTest):
         t = create_mock_topology(replica_set_name='rs')
         t.description.check_compatible()  # No error.
 
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a']})
 
@@ -570,9 +574,9 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(server.description.min_wire_version, 0)
         self.assertEqual(server.description.max_wire_version, 0)
 
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a'],
             'minWireVersion': 1,
@@ -582,13 +586,13 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(server.description.max_wire_version, 5)
 
         # Incompatible.
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a'],
-            'minWireVersion': 11,
-            'maxWireVersion': 12})
+            'minWireVersion': 21,
+            'maxWireVersion': 22})
 
         try:
             t.select_servers(any_server_selector)
@@ -596,16 +600,16 @@ class TestMultiServerTopology(TopologyTest):
             # Error message should say which server failed and why.
             self.assertEqual(
                 str(e),
-                "Server at a:27017 requires wire version 11, but this version "
+                "Server at a:27017 requires wire version 21, but this version "
                 "of PyMongo only supports up to %d."
                 % (common.MAX_SUPPORTED_WIRE_VERSION,))
         else:
             self.fail('No error with incompatible wire version')
 
         # Incompatible.
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a'],
             'minWireVersion': 0,
@@ -631,17 +635,17 @@ class TestMultiServerTopology(TopologyTest):
             s = t.select_server(writable_server_selector)
             return s.description.max_write_batch_size
 
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
             'maxWireVersion': 6,
             'maxWriteBatchSize': 1})
 
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
@@ -652,9 +656,9 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(1, write_batch_size())
 
         # b becomes primary.
-        got_ismaster(t, ('b', 27017), {
+        got_hello(t, ('b', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
             'maxWireVersion': 6,
@@ -665,9 +669,9 @@ class TestMultiServerTopology(TopologyTest):
     def test_topology_repr(self):
         t = create_mock_topology(replica_set_name='rs')
         self.addCleanup(t.close)
-        got_ismaster(t, ('a', 27017), {
+        got_hello(t, ('a', 27017), {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a', 'c', 'b']})
         self.assertEqual(
@@ -680,80 +684,100 @@ class TestMultiServerTopology(TopologyTest):
             "<ServerDescription ('c', 27017) server_type: Unknown,"
             " rtt: None>]>" % (t._topology_id,))
 
+    def test_unexpected_load_balancer(self):
+        # Note: This behavior should not be reachable in practice but we
+        # should handle it gracefully nonetheless. See PYTHON-2791.
+        # Load balancers are included in topology with a single seed.
+        t = create_mock_topology(seeds=['a'])
+        mock_lb_response = {'ok': 1, 'msg': 'isdbgrid',
+                            'serviceId': ObjectId(), 'maxWireVersion': 13}
+        got_hello(t, ('a', 27017), mock_lb_response)
+        sds = t.description.server_descriptions()
+        self.assertIn(('a', 27017), sds)
+        self.assertEqual(sds[('a', 27017)].server_type_name, 'LoadBalancer')
+        self.assertEqual(t.description.topology_type_name, 'Single')
+        self.assertTrue(t.description.has_writable_server())
 
-def wait_for_master(topology):
+        # Load balancers are removed from a topology with multiple seeds.
+        t = create_mock_topology(seeds=['a', 'b'])
+        got_hello(t, ('a', 27017), mock_lb_response)
+        self.assertNotIn(('a', 27017), t.description.server_descriptions())
+        self.assertEqual(t.description.topology_type_name, 'Unknown')
+
+
+def wait_for_primary(topology):
     """Wait for a Topology to discover a writable server.
 
-    If the monitor is currently calling ismaster, a blocking call to
+    If the monitor is currently calling hello, a blocking call to
     select_server from this thread can trigger a spurious wake of the monitor
     thread. In applications this is harmless but it would break some tests,
     so we pass server_selection_timeout=0 and poll instead.
     """
 
-    def get_master():
+    def get_primary():
         try:
             return topology.select_server(writable_server_selector, 0)
         except ConnectionFailure:
             return None
 
-    return wait_until(get_master, 'find master')
+    return wait_until(get_primary, 'find primary')
 
 
 class TestTopologyErrors(TopologyTest):
-    # Errors when calling ismaster.
+    # Errors when calling hello.
 
     def test_pool_reset(self):
-        # ismaster succeeds at first, then always raises socket error.
-        ismaster_count = [0]
+        # hello succeeds at first, then always raises socket error.
+        hello_count = [0]
 
         class TestMonitor(Monitor):
             def _check_with_socket(self, *args, **kwargs):
-                ismaster_count[0] += 1
-                if ismaster_count[0] == 1:
-                    return IsMaster({'ok': 1, 'maxWireVersion': 6}), 0
+                hello_count[0] += 1
+                if hello_count[0] == 1:
+                    return Hello({'ok': 1, 'maxWireVersion': 6}), 0
                 else:
                     raise AutoReconnect('mock monitor error')
 
         t = create_mock_topology(monitor_class=TestMonitor)
         self.addCleanup(t.close)
-        server = wait_for_master(t)
-        self.assertEqual(1, ismaster_count[0])
-        generation = server.pool.generation
+        server = wait_for_primary(t)
+        self.assertEqual(1, hello_count[0])
+        generation = server.pool.gen.get_overall()
 
-        # Pool is reset by ismaster failure.
+        # Pool is reset by hello failure.
         t.request_check_all()
-        self.assertNotEqual(generation, server.pool.generation)
+        self.assertNotEqual(generation, server.pool.gen.get_overall())
 
-    def test_ismaster_retry(self):
-        # ismaster succeeds at first, then raises socket error, then succeeds.
-        ismaster_count = [0]
+    def test_hello_retry(self):
+        # hello succeeds at first, then raises socket error, then succeeds.
+        hello_count = [0]
 
         class TestMonitor(Monitor):
             def _check_with_socket(self, *args, **kwargs):
-                ismaster_count[0] += 1
-                if ismaster_count[0] in (1, 3):
-                    return IsMaster({'ok': 1, 'maxWireVersion': 6}), 0
+                hello_count[0] += 1
+                if hello_count[0] in (1, 3):
+                    return Hello({'ok': 1, 'maxWireVersion': 6}), 0
                 else:
                     raise AutoReconnect(
-                        'mock monitor error #%s' % (ismaster_count[0],))
+                        'mock monitor error #%s' % (hello_count[0],))
 
         t = create_mock_topology(monitor_class=TestMonitor)
         self.addCleanup(t.close)
-        server = wait_for_master(t)
-        self.assertEqual(1, ismaster_count[0])
+        server = wait_for_primary(t)
+        self.assertEqual(1, hello_count[0])
         self.assertEqual(SERVER_TYPE.Standalone,
                          server.description.server_type)
 
-        # Second ismaster call, server is marked Unknown, then the monitor
-        # immediately runs a retry (third ismaster).
+        # Second hello call, server is marked Unknown, then the monitor
+        # immediately runs a retry (third hello).
         t.request_check_all()
-        # The third ismaster call (the immediate retry) happens sometime soon
+        # The third hello call (the immediate retry) happens sometime soon
         # after the failed check triggered by request_check_all. Wait until
         # the server becomes known again.
         server = t.select_server(writable_server_selector, 0.250)
         self.assertEqual(SERVER_TYPE.Standalone,
                          server.description.server_type)
-        self.assertEqual(3, ismaster_count[0])
+        self.assertEqual(3, hello_count[0])
 
     def test_internal_monitor_error(self):
         exception = AssertionError('internal error')
@@ -778,9 +802,9 @@ class TestServerSelectionErrors(TopologyTest):
 
     def test_no_primary(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a']})
@@ -793,9 +817,9 @@ class TestServerSelectionErrors(TopologyTest):
 
     def test_no_secondary(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': True,
+            HelloCompat.LEGACY_CMD: True,
             'setName': 'rs',
             'hosts': ['a']})
 
@@ -812,9 +836,9 @@ class TestServerSelectionErrors(TopologyTest):
 
     def test_bad_replica_set_name(self):
         t = create_mock_topology(replica_set_name='rs')
-        got_ismaster(t, address, {
+        got_hello(t, address, {
             'ok': 1,
-            'ismaster': False,
+            HelloCompat.LEGACY_CMD: False,
             'secondary': True,
             'setName': 'wrong',
             'hosts': ['a']})
@@ -825,8 +849,8 @@ class TestServerSelectionErrors(TopologyTest):
     def test_multiple_standalones(self):
         # Standalones are removed from a topology with multiple seeds.
         t = create_mock_topology(seeds=['a', 'b'])
-        got_ismaster(t, ('a', 27017), {'ok': 1})
-        got_ismaster(t, ('b', 27017), {'ok': 1})
+        got_hello(t, ('a', 27017), {'ok': 1})
+        got_hello(t, ('b', 27017), {'ok': 1})
         self.assertMessage('No servers available', t)
 
     def test_no_mongoses(self):
@@ -834,11 +858,11 @@ class TestServerSelectionErrors(TopologyTest):
         t = create_mock_topology(seeds=['a', 'b'])
 
         # Discover a mongos and change topology type to Sharded.
-        got_ismaster(t, ('a', 27017), {'ok': 1, 'msg': 'isdbgrid'})
+        got_hello(t, ('a', 27017), {'ok': 1, 'msg': 'isdbgrid'})
 
         # Oops, both servers are standalone now. Remove them.
-        got_ismaster(t, ('a', 27017), {'ok': 1})
-        got_ismaster(t, ('b', 27017), {'ok': 1})
+        got_hello(t, ('a', 27017), {'ok': 1})
+        got_hello(t, ('b', 27017), {'ok': 1})
         self.assertMessage('No mongoses available', t)
 
 
