@@ -19,15 +19,21 @@ import sys
 
 sys.path[0:0] = [""]
 
+from bson.py3compat import StringIO
+
 from pymongo import client_session, WriteConcern
 from pymongo.client_session import TransactionOptions
-from pymongo.errors import (ConfigurationError,
+from pymongo.errors import (CollectionInvalid,
+                            ConfigurationError,
                             ConnectionFailure,
+                            InvalidOperation,
                             OperationFailure)
 from pymongo.operations import IndexModel, InsertOne
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
 
+from gridfs import GridFS, GridFSBucket
+
 from test import unittest, client_context
 from test.utils import (rs_client, single_client,
                         wait_until, OvertCommandListener,
@@ -35,8 +41,8 @@ from test.utils import (rs_client, single_client,
 from test.utils_spec_runner import SpecRunner
 
 # Location of JSON test specifications.
-_TEST_PATH = os.path.join(
-    os.path.dirname(os.path.realpath(__file__)), 'transactions')
+TEST_PATH = os.path.join(
+    os.path.dirname(os.path.realpath(__file__)), 'transactions', 'legacy')
 
 _TXN_TESTS_DEBUG = os.environ.get('TRANSACTION_TESTS_DEBUG')
 
@@ -55,6 +61,12 @@ class TransactionsBase(SpecRunner):
             for address in client_context.mongoses:
                 cls.mongos_clients.append(single_client('%s:%s' % address))
 
+    @classmethod
+    def tearDownClass(cls):
+        for client in cls.mongos_clients:
+            client.close()
+        super(TransactionsBase, cls).tearDownClass()
+
     def maybe_skip_scenario(self, test):
         super(TransactionsBase, self).maybe_skip_scenario(test)
         if ('secondary' in self.id() and
@@ -64,6 +76,7 @@ class TransactionsBase(SpecRunner):
 
 
 class TestTransactions(TransactionsBase):
+    RUN_ON_SERVERLESS = True
     @client_context.require_transactions
     def test_transaction_options_validation(self):
         default_options = TransactionOptions()
@@ -91,7 +104,6 @@ class TestTransactions(TransactionsBase):
                 TypeError, "max_commit_time_ms must be an integer or None"):
             TransactionOptions(max_commit_time_ms="10000")
 
-
     @client_context.require_transactions
     def test_transaction_write_concern_override(self):
         """Test txn overrides Client/Database/Collection write_concern."""
@@ -121,7 +133,6 @@ class TestTransactions(TransactionsBase):
 
         unsupported_txn_writes = [
             (client.drop_database, [db.name], {}),
-            (db.create_collection, ['collection'], {}),
             (db.drop_collection, ['collection'], {}),
             (coll.drop, [], {}),
             (coll.map_reduce,
@@ -135,6 +146,12 @@ class TestTransactions(TransactionsBase):
             (coll.drop_indexes, [], {}),
             (coll.aggregate, [[{"$out": "aggout"}]], {}),
         ]
+        # Creating a collection in a transaction requires MongoDB 4.4+.
+        if client_context.version < (4, 3, 4):
+            unsupported_txn_writes.extend([
+                (db.create_collection, ['collection'], {}),
+            ])
+
         for op in unsupported_txn_writes:
             op, args, kwargs = op
             with client.start_session() as s:
@@ -201,6 +218,114 @@ class TestTransactions(TransactionsBase):
 
             self.assertGreater(len(addresses), 1)
 
+    @client_context.require_transactions
+    @client_context.require_version_min(4, 3, 4)
+    def test_create_collection(self):
+        client = client_context.client
+        db = client.pymongo_test
+        coll = db.test_create_collection
+        self.addCleanup(coll.drop)
+
+        # Use with_transaction to avoid StaleConfig errors on sharded clusters.
+        def create_and_insert(session):
+            coll2 = db.create_collection(coll.name, session=session)
+            self.assertEqual(coll, coll2)
+            coll.insert_one({}, session=session)
+
+        with client.start_session() as s:
+            s.with_transaction(create_and_insert)
+
+        # Outside a transaction we raise CollectionInvalid on existing colls.
+        with self.assertRaises(CollectionInvalid):
+            db.create_collection(coll.name)
+
+        # Inside a transaction we raise the OperationFailure from create.
+        with client.start_session() as s:
+            s.start_transaction()
+            with self.assertRaises(OperationFailure) as ctx:
+                db.create_collection(coll.name, session=s)
+            self.assertEqual(ctx.exception.code, 48)  # NamespaceExists
+
+    @client_context.require_transactions
+    def test_gridfs_does_not_support_transactions(self):
+        client = client_context.client
+        db = client.pymongo_test
+        gfs = GridFS(db)
+        bucket = GridFSBucket(db)
+
+        def gridfs_find(*args, **kwargs):
+            return gfs.find(*args, **kwargs).next()
+
+        def gridfs_open_upload_stream(*args, **kwargs):
+            bucket.open_upload_stream(*args, **kwargs).write(b'1')
+
+        gridfs_ops = [
+            (gfs.put, (b'123',)),
+            (gfs.get, (1,)),
+            (gfs.get_version, ('name',)),
+            (gfs.get_last_version, ('name',)),
+            (gfs.delete, (1, )),
+            (gfs.list, ()),
+            (gfs.find_one, ()),
+            (gridfs_find, ()),
+            (gfs.exists, ()),
+            (gridfs_open_upload_stream, ('name',)),
+            (bucket.upload_from_stream, ('name', b'data',)),
+            (bucket.download_to_stream, (1, StringIO(),)),
+            (bucket.download_to_stream_by_name, ('name', StringIO(),)),
+            (bucket.delete, (1,)),
+            (bucket.find, ()),
+            (bucket.open_download_stream, (1,)),
+            (bucket.open_download_stream_by_name, ('name',)),
+            (bucket.rename, (1, 'new-name',)),
+        ]
+
+        with client.start_session() as s, s.start_transaction():
+            for op, args in gridfs_ops:
+                with self.assertRaisesRegex(
+                        InvalidOperation,
+                        'GridFS does not support multi-document transactions',
+                ):
+                    op(*args, session=s)
+
+    # Require 4.2+ for large (16MB+) transactions.
+    @client_context.require_version_min(4, 2)
+    @client_context.require_transactions
+    @unittest.skipIf(sys.platform.startswith('java'),
+                     'Jython is too slow to pass this test')
+    @unittest.skipIf(sys.platform == 'win32',
+                     'Our Windows machines are too slow to pass this test')
+    def test_transaction_starts_with_batched_write(self):
+        if 'PyPy' in sys.version and client_context.tls:
+            self.skipTest('PYTHON-2937 PyPy is so slow sending large '
+                          'messages over TLS that this test fails')
+        # Start a transaction with a batch of operations that needs to be
+        # split.
+        listener = OvertCommandListener()
+        client = rs_client(event_listeners=[listener])
+        coll = client[self.db.name].test
+        coll.delete_many({})
+        listener.reset()
+        self.addCleanup(client.close)
+        self.addCleanup(coll.drop)
+        large_str = '\0'*(10*1024*1024)
+        ops = [InsertOne({'a': large_str}) for _ in range(10)]
+        with client.start_session() as session:
+            with session.start_transaction():
+                coll.bulk_write(ops, session=session)
+        # Assert commands were constructed properly.
+        self.assertEqual(['insert', 'insert', 'insert', 'commitTransaction'],
+                         listener.started_command_names())
+        first_cmd = listener.results['started'][0].command
+        self.assertTrue(first_cmd['startTransaction'])
+        lsid = first_cmd['lsid']
+        txn_number = first_cmd['txnNumber']
+        for event in listener.results['started'][1:]:
+            self.assertNotIn('startTransaction', event.command)
+            self.assertEqual(lsid, event.command['lsid'])
+            self.assertEqual(txn_number, event.command['txnNumber'])
+        self.assertEqual(10, coll.count_documents({}))
+
 
 class PatchSessionTimeout(object):
     """Patches the client_session's with_transaction timeout for testing."""
@@ -249,6 +374,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
     def test_callback_not_retried_after_timeout(self):
         listener = OvertCommandListener()
         client = rs_client(event_listeners=[listener])
+        self.addCleanup(client.close)
         coll = client[self.db.name].test
 
         def callback(session):
@@ -278,6 +404,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
     def test_callback_not_retried_after_commit_timeout(self):
         listener = OvertCommandListener()
         client = rs_client(event_listeners=[listener])
+        self.addCleanup(client.close)
         coll = client[self.db.name].test
 
         def callback(session):
@@ -308,6 +435,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
     def test_commit_not_retried_after_timeout(self):
         listener = OvertCommandListener()
         client = rs_client(event_listeners=[listener])
+        self.addCleanup(client.close)
         coll = client[self.db.name].test
 
         def callback(session):
@@ -380,7 +508,7 @@ def create_test(scenario_def, test, name):
     return run_scenario
 
 
-test_creator = TestCreator(create_test, TestTransactions, _TEST_PATH)
+test_creator = TestCreator(create_test, TestTransactions, TEST_PATH)
 test_creator.create_tests()
 
 
